@@ -256,15 +256,21 @@ def get_heatmap_color(color_map, min_val, max_val, values, zscale='log'):
 def timebar_delete(t, varname=None, dim='height'):
     if varname is None:
         for name in pytplot.data_quants:
-            list_timebars = pytplot.data_quants[name]['plot_options']['time_bar']
+            if isinstance(pytplot.data_quants[name], dict):
+                # non-record varying variable
+                continue
+            try:
+                list_timebars = pytplot.data_quants[name].attrs['plot_options']['time_bar']
+            except KeyError:
+                continue
             elem_to_delete = []
             for elem in list_timebars:
                 for num in t:
-                    if (elem.location == num) and (elem.dimension == dim):
+                    if (elem['location'] == num) and (elem['dimension'] == dim):
                         elem_to_delete.append(elem)
             for i in elem_to_delete:
                 list_timebars.remove(i)
-            pytplot.data_quants[name]['plot_options']['time_bar'] = list_timebars
+            pytplot.data_quants[name].attrs['plot_options']['time_bar'] = list_timebars
     else:
         if not isinstance(varname, list):
             varname = [varname]
@@ -272,15 +278,21 @@ def timebar_delete(t, varname=None, dim='height'):
             if i not in pytplot.data_quants.keys():
                 print(str(i) + " is currently not in pytplot.")
                 return
-            list_timebars = pytplot.data_quants[i]['plot_options']['time_bar']
+            if isinstance(pytplot.data_quants[i], dict):
+                # non-record varying variable
+                continue
+            try:
+                list_timebars = pytplot.data_quants[i].attrs['plot_options']['time_bar']
+            except KeyError:
+                continue
             elem_to_delete = []
             for elem in list_timebars:
                 for num in t:
-                    if (elem.location == num) and (elem.dimension == dim):
+                    if (elem['location'] == num) and (elem['dimension'] == dim):
                         elem_to_delete.append(elem)
             for j in elem_to_delete:
                 list_timebars.remove(j)
-            pytplot.data_quants[i]['plot_options']['time_bar']
+            # pytplot.data_quants[i].attrs['plot_options']['time_bar']
     return
 
 
@@ -684,3 +696,66 @@ def reduce_spec_dataset(tplot_dataset=None, name=None):
             else:
                 da = da.sum(dim=d, skipna=True, keep_attrs=True)
     return da
+
+
+def highlight(variables, range, color='gray', alpha=0.2, fill=True, edgecolor=None, facecolor=None, hatch=None):
+    """
+    This function highlights a time interval on tplot variables
+
+    """
+    if not isinstance(variables, list):
+        variables = [variables]
+
+    for variable in variables:
+        if range is None:
+            pytplot.data_quants[variable].attrs['plot_options']['highlight_intervals'] = None
+            return
+        interval = {'location': range,
+                    'color': color,
+                    'alpha': alpha,
+                    'fill': fill,
+                    'edgecolor': edgecolor,
+                    'facecolor': facecolor,
+                    'hatch': hatch}
+        if pytplot.data_quants[variable].attrs['plot_options'].get('highlight_intervals') is None:
+            pytplot.data_quants[variable].attrs['plot_options']['highlight_intervals'] = [interval]
+        else:
+            pytplot.data_quants[variable].attrs['plot_options']['highlight_intervals'].append(interval)
+
+
+def annotate(text=None, position=None,
+             xycoords='figure fraction',
+             color='black',
+             fontfamily=None,
+             fontsize='x-large',
+             alpha=1,
+             fontvariant='normal',
+             fontstyle='normal',
+             fontstretch='normal',
+             fontweight='normal',
+             rotation='horizontal',
+             delete=False):
+    """
+    This function adds annotations to tplot figures
+    """
+    if delete:
+        pytplot.tplot_opt_glob['annotations'] = None
+        return
+
+    annotations = {'text': text,
+                   'position': position,
+                   'xycoords': xycoords,
+                   'fontfamily': fontfamily,
+                   'fontsize': fontsize,
+                   'fontvariant': fontvariant,
+                   'fontstyle': fontstyle,
+                   'fontstretch': fontstretch,
+                   'fontweight': fontweight,
+                   'rotation': rotation,
+                   'color': color,
+                   'alpha': alpha}
+
+    if pytplot.tplot_opt_glob.get('annotations') is None:
+        pytplot.tplot_opt_glob['annotations'] = [annotations]
+    else:
+        pytplot.tplot_opt_glob['annotations'].append(annotations)
\ No newline at end of file
